import os
import sys

def seed_everything(seed):
    import torch
    import numpy as np
    import random
    torch.manual_seed(seed)       # Current CPU
    torch.cuda.manual_seed(seed)  # Current GPU
    np.random.seed(seed)          # Numpy module
    random.seed(seed)             # Python random module
    torch.backends.cudnn.benchmark = False    # Close optimization
    torch.backends.cudnn.deterministic = True # Close optimization
    # torch.cuda.manual_seed_all(seed) # All GPU (Optional)

def set_process_gpu():
    import torch
    worker_id = int(os.environ.get('APP_WORKER_ID', 1))
    devices = os.environ.get('CUDA_VISIBLE_DEVICES', '')
 
    if not devices:
        print('current environment did not get CUDA_VISIBLE_DEVICES env ,so use the default')
        return torch.device("cpu")
    rand_max = 9527
    gpu_index = (worker_id + rand_max) % torch.cuda.device_count()
    res = f"pid: {os.getpid()}, worker_id: {worker_id} set gpu_id :{gpu_index}"
    torch.cuda.set_device(int(gpu_index))
    print(res)
    return torch.device(gpu_index)


def try_occupy_more_gpu_memory(max_mb, device):
    import torch
    N = 0
    L = []
    X = torch.zeros(250000, 1000)
    if max_mb < 1000:
        try:
            x = X.clone().to(device)
            del x
            print(f"预先使用{max_mb} MB gpu")
        except Exception as e:
            print(e)
    else:
        while N < max_mb:
            try:
                L.append(X.clone().to(device))
                N += 1000
            except Exception as e:
                print(e)
                for l in L:
                    del l
                break

        for l in L:
            del l
    print(f"N: {N} MB")


from motti.standard import (
    pil2str,
    str2pil,
    get_datetime,
    init_project_dir_with,
    is_abs_path,
    load_yaml,
    load_namespace_from_yaml,
    create_namespace,
    pt_to_pil,
    numpy_to_pil,
    is_video_file,
    is_image_file,
    is_document_file,
    o_d,
    mkdir_or_exist,
    uint8_imread,
    load_json,
    dump_json,
    append_current_dir,
    append_parent_dir,
    figure2pil,
)

from motti.extension import seed_everything





from io import BytesIO
import PIL
from datetime import datetime
import os
import sys
from typing import Optional, Union
from pathlib import Path
import argparse
import numpy as np
import io
import matplotlib.pyplot as plt
import base64
import yaml
import json

def uint8_imread(path: str):
    img = plt.imread(path)
    if img.dtype != np.uint8 and img.max() <= 1.0:
        img = img * 255
        img = img.astype(np.uint8)
    return img


def pil2str(x):
    buffer = BytesIO()
    x.save(buffer, format='PNG')
    b64 = base64.b64encode(buffer.getvalue())
    res = str(b64, 'utf-8')
    return res


def str2pil(s):
    b64 = base64.b64decode(s.encode('utf-8'))
    return PIL.Image.open(BytesIO(b64))


def get_datetime():
    return datetime.now().strftime("%Y%m%d-%H%M%S")


def init_project_dir_with(project_dir="."):
    def f(*args, **kwargs):
        return os.path.join(project_dir, *args, **kwargs)
    return f


def is_abs_path(path: Union[Path, str]):
    if isinstance(path, Path):
        return path.is_absolute()
    elif isinstance(path, str):
        return os.path.isabs(path)
    else:
        raise NotImplementedError 
        return False


def load_yaml(path):
    return yaml.safe_load(open(path, 'r'))


def load_namespace_from_yaml(path):
    D = load_yaml(path)
    return argparse.Namespace(**D)


def create_namespace(D:dict):
    return argparse.Namespace(**D)


def pt_to_pil(images):
    """
    Convert a torch image to a PIL image.
    """
    images = (images / 2 + 0.5).clamp(0, 1)
    images = images.cpu().permute(0, 2, 3, 1).float().numpy()
    images = numpy_to_pil(images)
    return images


def numpy_to_pil(images):
    """
    Convert a numpy image or a batch of images to a PIL image.
    """
    if images.ndim == 3:
        images = images[None, ...]
    images = (images * 255).round().astype("uint8")
    if images.shape[-1] == 1:
        # special case for grayscale (single channel) images
        pil_images = [Image.fromarray(image.squeeze(), mode="L") for image in images]
    else:
        pil_images = [Image.fromarray(image) for image in images]

    return pil_images


def is_video_file(file_name):
    video_extensions = ['.mp4', '.avi', '.mov', '.mkv', '.wmv']  # 添加其他视频文件格式
    return any(file_name.lower().endswith(ext) for ext in video_extensions)


def is_image_file(file_name):
    photo_extensions = ['.jpg', '.jpeg', '.png', '.bmp', '.webp']  # 添加其他照片文件格式
    # 将文件名后缀转换为小写，并检查是否在照片文件后缀列表中
    return any(file_name.lower().endswith(ext) for ext in photo_extensions)


def is_document_file(file_name):
    document_extensions = ['.doc', '.docx', '.pdf', '.txt', '.odt']  # 添加其他文档文件格式

    # 将文件名后缀转换为小写，并检查是否在文档文件后缀列表中
    return any(file_name.lower().endswith(ext) for ext in document_extensions)


def o_d():
    return datetime.now().strftime("%Y%m%d-%H%M%S")


def mkdir_or_exist(dir_name, mode=0o777):
    if dir_name == '':
        return
    dir_name = os.path.expanduser(dir_name)
    os.makedirs(dir_name, mode=mode, exist_ok=True)


def append_current_dir(filepath):
    current_dir = os.path.dirname(os.path.abspath(filepath))
    sys.path.append(current_dir)
    return current_dir


def append_parent_dir(filepath):
    current_dir = os.path.dirname(os.path.abspath(filepath))
    parent_dir = os.path.dirname(current_dir)
    sys.path.append(parent_dir)
    return parent_dir


def load_json(path):
    with open(path, "r") as f:
        return json.load(f)
    
    
def dump_json(d, path):
    with open(path, "w") as f:
        return json.dump(d, f, indent=4)


def pt2numpy(images):
    images.cpu().permute(0, 2, 3, 1).float().numpy()
    return images


def figure2pil(fig):
    buf = io.BytesIO()
    fig.savefig(buf, bbox_inches="tight", pad_inches=0.1)
    buf.seek(0)
    return PIL.Image.open(buf)



import os
import logging
import time

def init_logger(
        logdir: str="log",
        logname: str="root"
    ) -> logging.Logger:
        logger = logging.getLogger(logname)
        logger.setLevel(logging.DEBUG)

        os.makedirs(logdir, exist_ok=True)
        # filename = time.strftime("%Y%m%d-%H%M%S", time.localtime(time.time())) + ".log"
        filename = os.uname()[1] + "_" + time.strftime("%Y%m%d", time.localtime(time.time())) + ".log"
        log_path = os.path.join(logdir, filename)

        fh = logging.FileHandler(log_path)
        fh.setLevel(logging.INFO)

        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        formatter = logging.Formatter(
            "%(asctime)s - " \
            "%(process)d - " \
            "%(name)s - " \
            "%(levelname)s - " \
            "%(message)s"
        )

        fh.setFormatter(formatter)
        ch.setFormatter(formatter)

        logger.addHandler(fh)
        logger.addHandler(ch)
        return logger

def get_the_most_idle_gpu_index_and_free_memory():
    import pynvml
    pynvml.nvmlInit()  # 初始化

    def f(threshold=8000):
        gpu_device_count = pynvml.nvmlDeviceGetCount()  # 获取Nvidia GPU块数

        return_gpu_index = 0
        max_free_memory = -1

        for gpu_index in range(gpu_device_count):
            handle = pynvml.nvmlDeviceGetHandleByIndex(gpu_index)  # 获取GPU i的handle，后续通过handle来处理
            memery_info = pynvml.nvmlDeviceGetMemoryInfo(handle)  # 通过handle获取GPU 的信息
            if memery_info.free > max_free_memory:
                max_free_memory = memery_info.free
                return_gpu_index = gpu_index

        if (max_free_memory>>20) > threshold:
            return return_gpu_index
        else:
            return -1
    return f


def set_process_gpu():
    import torch
    worker_id = int(os.environ.get('APP_WORKER_ID', 1))
    devices = os.environ.get('CUDA_VISIBLE_DEVICES', '')
 
    if not devices:
        print('current environment did not get CUDA_VISIBLE_DEVICES env ,so use the default')
        return 0

    rand_max = 9527
    gpu_index = (worker_id + rand_max) % torch.cuda.device_count()
    torch.cuda.set_device(int(gpu_index))
    res = f"pid: {os.getpid()}, worker_id: {worker_id} set gpu_id :{gpu_index}"
    print(res)
    return gpu_index



from setuptools import find_packages, setup

setup(
    name="motti",
    version="0.0.36",
    author="lup1n",
    author_email="780966523@qq.com",
    description="some util functions",
    packages=find_packages(),
    install_requires=[
        "numpy",
        "pillow",
        "matplotlib",
        "pybase64",
        "pyyaml",
    ],
)